/*-
 * %%----------------------------------------------------------------------------------------------
 * Archive ouverte UNIGE - AoU Access - PublicationStatisticService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2022 - 2023 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.unige.aou.service;

import java.time.LocalDateTime;
import java.util.List;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.stereotype.Service;

import ch.unige.aou.controller.AccessController;
import ch.unige.aou.model.access.PublicationDownload;
import ch.unige.aou.model.access.StatisticsInfoDto;
import ch.unige.aou.model.index.PublicationIndexEntry;
import ch.unige.aou.model.publication.Download;
import ch.unige.aou.model.publication.Hit;
import ch.unige.aou.repository.DownloadRepository;
import ch.unige.aou.repository.HitRepository;

@Service
@ConditionalOnBean(AccessController.class)
public class PublicationStatisticService {

  private final DownloadRepository downloadRepository;
  private final HitRepository hitRepository;

  public PublicationStatisticService(DownloadRepository downloadRepository, HitRepository hitRepository) {
    this.downloadRepository = downloadRepository;
    this.hitRepository = hitRepository;
  }

  public Download createDownloadStats(String documentFileId, PublicationDownload document) {
    LocalDateTime now = LocalDateTime.now();
    Download documentStats = this.downloadRepository.findByDocumentFileAndYearAndMonth(documentFileId, now.getYear(), now.getMonthValue())
            .orElse(null);
    if (documentStats == null) {
      // create new record
      Download download = new Download();
      download.setDownloadNumber(1);
      download.setFilename(document.getName());
      download.setDocumentFileId(document.getResId());
      download.setPublicationId(document.getPublicationId());
      download.setYear(now.getYear());
      download.setMonth(now.getMonthValue());
      return this.downloadRepository.save(download);
    } else {
      // append additional downloadNumber for the current month
      documentStats.setDownloadNumber(documentStats.getDownloadNumber() + 1);
      return this.downloadRepository.save(documentStats);
    }
  }

  public Hit createViewStats(PublicationIndexEntry publication) {
    Hit existingHit = this.hitRepository.findByPublicationId(publication.getResId()).orElse(null);
    if (existingHit != null) {
      existingHit.setViewNumber(existingHit.getViewNumber() + 1);
      return this.hitRepository.save(existingHit);
    } else {
      Hit newHit = new Hit();
      newHit.setPublicationId(publication.getResId());
      newHit.setArchiveId((String) publication.getMetadata().get("archiveId"));
      newHit.setType((String) publication.getMetadata().get("type"));
      newHit.setViewNumber(1);
      return this.hitRepository.save(newHit);
    }
  }

  public StatisticsInfoDto getPublicationStatistics(String publicationId) {
    Hit hit = this.hitRepository.findByPublicationId(publicationId).orElse(null);
    Integer viewNumber = hit != null ? hit.getViewNumber() : 0;
    List<Download> downloadList = this.downloadRepository.findByPublicationId(publicationId);
    return new StatisticsInfoDto(viewNumber, downloadList.stream().map(Download::getDownloadNumber).reduce(0, Integer::sum));
  }
}
